import os

from .STDR import STDR
from .ATR import ATR
from .BPS import BPS
from .DTR import DTR
from .EPS import EPS
from .FAR import FAR
from .FTR import FTR
from .GDR import GDR
from .HBR import HBR
from .MIR import MIR
from .MPR import MPR
from .MRR import MRR
from .PCR import PCR
from .PGR import PGR
from .PIR import PIR
from .PLR import PLR
from .PMR import PMR
from .PRR import PRR
from .PTR import PTR
from .RDR import RDR
from .SBR import SBR
from .SDR import SDR
from .TSR import TSR
from .WCR import WCR
from .WIR import WIR
from .WRR import WRR

__version__ = '0.0.0'
__version__='0.1.5'
